@@ -1,4 +1,3 @@
-from importlib.resources import open_text
 from logging.config import dictConfig
 from raft import Collection, Program
 from . import sewer
@@ -44,4 +43,4 @@ dictConfig(dict(
 ))
 ns = Collection.from_module(sewer)
 ns.add_collection(installers.installers_collection, 'install')
-program = Program(version=open_text('sedge', 'version.txt').read(), namespace=ns)
+program = Program(version='1.5.3', namespace=ns)
