@@ -5,6 +5,7 @@ import os
 import posixpath
 import socket
 import sys
+from tempfile import NamedTemporaryFile
 from typing import Sequence, Dict
 
 from raft import task
@@ -221,11 +222,13 @@ def get_chunked_ssm_parameter(name, profile=None):
     return data
 
 
-def get_pfx(ns, hostname, bucket, profile=None):
+def get_pfx(bucket, key, profile=None):
     session = Session(profile_name=profile)
     s3 = session.client('s3')
-    hostname = hostname.replace('*', 'star')
-    pfx_data = get_file_from_s3(s3, bucket, ns, f'{hostname}.pfx', False)
+    key = key.replace('*', 'star')
+    if not key.lower().endswith('.pfx'):
+        key = f'{key}.pfx'
+    pfx_data = get_file_from_s3(s3, bucket, None, key, False)
     log.info('[pfx]  read %s bytes', len(pfx_data))
     return pfx_data
 
@@ -300,24 +303,31 @@ def pfx(ctx, certificate, key):
     return data
 
 
-def full_pfx(ctx, certificate, key, tmp_dir='/tmp'):
-    f1 = os.path.join(tmp_dir, 'p1.crt')
-    f2 = os.path.join(tmp_dir, 'p1.key')
-    f3 = os.path.join(tmp_dir, 'p1.pfx')
-    with open(f1, 'w') as f:
+def full_pfx(ctx, certificate, key, tmp_dir='/tmp', password=None):
+    cert = NamedTemporaryFile(mode='w', delete=False)  # pylint: disable=consider-using-with
+    key = NamedTemporaryFile(mode='w', delete=False)  # pylint: disable=consider-using-with
+    f_pfx = NamedTemporaryFile(mode='wb', delete=False)  # pylint: disable=consider-using-with
+    with open(cert.name, 'w') as f:
         f.write(certificate)
-    with open(f2, 'w') as f:
+    with open(key.name, 'w') as f:
         f.write(key)
-    os.chmod(f1, 0o644)
-    os.chmod(f2, 0o600)
-    ctx.run(
-        f'/usr/bin/openssl pkcs12 -export -in {f1} -inkey {f2}'
-        f' -out {f3} -passout pass:')
-    with open(f3, 'rb') as f:
+    os.chmod(cert.name, 0o644)
+    os.chmod(key.name, 0o600)
+    if not password:
+        ctx.run(
+            f'/usr/bin/openssl pkcs12 -export -in {cert.name} -inkey {key.name}'
+            f' -out {f_pfx.name} -passout pass:')
+    else:
+        ctx.run(
+            f'/usr/bin/openssl pkcs12 -export -in {cert.name} -inkey {key.name}'
+            f' -out {f_pfx.name} -passout env:SEDGE_PASSWORD',
+            env=dict(SEDGE_PASSWORD=password),
+        )
+    with open(f_pfx.name, 'rb') as f:
         data = f.read()
-    os.remove(f1)
-    os.remove(f2)
-    os.remove(f3)
+    os.remove(cert.name)
+    os.remove(key.name)
+    os.remove(f_pfx.name)
     return data
 
 
@@ -365,6 +375,7 @@ def request_cert(ctx, filename, profile, defaults):
         values.setdefault(key, value)
     namespaces = values.pop('namespaces', [])
     config_profile = values.pop('profile', None)
+    pfx_password = values.pop('pfx_password', None)
     profile = profile or config_profile
     ns = namespaces[0]
     certificate, account_key, key = renew_cert(
@@ -377,7 +388,8 @@ def request_cert(ctx, filename, profile, defaults):
             certificate, account_key, key)
         save_to_s3(
             ctx, bucket, x, values['hostname'], certificate,
-            account_key, key, tmp_dir=tmp_dir, profile=profile)
+            account_key, key, tmp_dir=tmp_dir, profile=profile,
+            pfx_password=pfx_password)
 
 
 @task
@@ -436,12 +448,12 @@ def save_to_file(ctx, tmp_dir, hostname, certificate, account_key, key):
 
 
 def save_to_s3(ctx, bucket, ns, hostname, certificate, account_key, key,
-               tmp_dir='/tmp', profile=None):
+               tmp_dir='/tmp', profile=None, pfx_password=None):
     """
     saves the contents of the certificate, key, and account keys
     to a local directory for debugging
     """
-    pfx_content = full_pfx(ctx, certificate, key)
+    pfx_content = full_pfx(ctx, certificate, key, pfx_password)
     contents = [
         ('.crt', certificate),
         ('.key', key),
@@ -545,6 +557,7 @@ def install_cert(ctx, config, hostname=None):
     cert_filename = conf.get('certificate')
     key_filename = conf.get('key')
     hostname = hostname or conf.get('hostname')
+    pfx_password = conf.get('pfx_password')
     bucket = conf.get('bucket')
     if not hostname:
         hostname = get_hostname(ctx)
@@ -553,8 +566,7 @@ def install_cert(ctx, config, hostname=None):
             ctx, ns, hostname, profile,
             cert_filename, key_filename, owner, group, bucket=bucket)
     elif is_windows():
-        install_cert_on_windows(
-            ctx, ns, hostname, profile)
+        install_cert_on_windows(ctx, bucket, hostname, profile, pfx_password)
 
 
 def get_hostname(ctx):
@@ -589,27 +601,79 @@ def install_cert_on_linux(
     ctx.run(f'chown {owner}:{group} {key_filename}')
 
 
-@task
-def install_cert_on_windows(ctx, ns, hostname, profile):
+def windows_version(ctx):
+    c = r"""
+    $key = 'HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion'
+    $x = Get-ItemProperty $key
+    Write-Host $x.ProductName
     """
-    not yet implemented -- have to find a good, cost-effective way
-    to generate the pfx file and store to ssm
-    """
-    pfx_data = get_pfx(ns, hostname, profile)
-    c = 'powershell.exe -command "[System.IO.Path]::GetTempFileName()"'
+    print('checking windows version')
     result = ctx.run(c)
-    filename = result.stdout.strip()
+    version = result.stdout.strip()
+    return version
+
+
+@task
+def install_cert_on_windows(
+        ctx, bucket, key, profile, pfx_password=None, store=None):
+    """
+    pull a file from s3 and install in the localmachine/my cert store
+    """
+    from cryptography.hazmat.primitives.serialization import pkcs12
+    from cryptography.hazmat.primitives import hashes
+    print('getting cert')
+    pfx_data = get_pfx(bucket, key, profile)
+    print('getting thumbprint')
+    _, cert, _ = pkcs12.load_key_and_certificates(pfx_data, pfx_password)
+    thumbprint = cert.fingerprint(hashes.SHA1()).hex().upper()
+    f = NamedTemporaryFile(mode='w', delete=False)  # pylint: disable=consider-using-with
+    filename = f.name
+    print('importing pfx to cert store')
     with open(filename, 'wb') as f:
         f.write(pfx_data)
-    c = (
-        'powershell.exe -command "'
-        'Import-PfxCertificate '
-        r'  -CertStoreLocation cert:\localmachine\my'
-        f' -filepath {filename}'
-        f'"'
-    )
-    ctx.run(c)
+    cmdlet = """
+    function Import-PfxCertificate {
+        param([string]$filename
+          , [string]$root_store = 'localmachine'
+          , [string]$store = 'My'
+          , [security.securestring]$password = $null)
+        $pfx = new-object Security.Cryptography.X509Certificates.X509Certificate2
+        $pfx.import($filename, $password, 'Exportable,PersistKeySet,MachineKeySet')
+        $p = new-object Security.Cryptography.X509Certificates.X509Store($store, $root_store)
+        $p.open('MaxAllowed')
+        $exists = $false
+        foreach ($x in $p.certificates) {
+            if ($x.thumbprint -eq $pfx.thumbprint) {
+                Write-Host 'cert already exists in cert store'
+                $exists = $true
+            }
+        }
+        if (!($exists)) {
+            $p.add($pfx)
+        }
+        $p.close()
+        return $pfx.thumbprint
+    }
+    """
+    password = ''
+    store_arg = ''
+    e = {}
+    if pfx_password:
+        password = "-password (ConvertTo-SecureString -f -a $env:SEDGE_PASSWORD)"
+        e['SEDGE_PASSWORD'] = pfx_password
+    if store:
+        store_arg = f"-store '{store}'"
+    lines = [
+        cmdlet,
+        f"$t = Import-PfxCertificate -filename '{filename}' {password} "
+        f"{store_arg}",
+        'Write-Host $t',
+    ]
+    c = '\n'.join(lines)
+    ctx.run(c, env=e)
+    print(f'removing {filename}')
     os.remove(filename)
+    return thumbprint
 
 
 def is_linux():
